@@ -19,9 +19,7 @@ package collector
 
 import (
 	"errors"
-
-	"github.com/go-kit/log"
-	"github.com/go-kit/log/level"
+	"log/slog"
 )
 
 /*
@@ -34,7 +32,7 @@ import (
 */
 import "C"
 
-func getNetDevStats(filter *deviceFilter, logger log.Logger) (netDevStats, error) {
+func getNetDevStats(filter *deviceFilter, logger *slog.Logger) (netDevStats, error) {
 	netDev := netDevStats{}
 
 	var ifap, ifa *C.struct_ifaddrs
@@ -50,7 +48,7 @@ func getNetDevStats(filter *deviceFilter, logger log.Logger) (netDevStats, error
 
 		dev := C.GoString(ifa.ifa_name)
 		if filter.ignored(dev) {
-			level.Debug(logger).Log("msg", "Ignoring device", "device", dev)
+			logger.Debug("Ignoring device", "device", dev)
 			continue
 		}
 
@@ -72,3 +70,7 @@ func getNetDevStats(filter *deviceFilter, logger log.Logger) (netDevStats, error
 
 	return netDev, nil
 }
+
+func getNetDevLabels() (map[string]map[string]string, error) {
+	return nil, nil
+}
