@@ -20,6 +20,7 @@ import (
 	"context"
 	"errors"
 	"fmt"
+	"log/slog"
 	"math"
 	"regexp"
 	"strconv"
@@ -27,11 +28,9 @@ import (
 	"sync"
 	"time"
 
+	"github.com/alecthomas/kingpin/v2"
 	"github.com/coreos/go-systemd/v22/dbus"
-	"github.com/go-kit/log"
-	"github.com/go-kit/log/level"
 	"github.com/prometheus/client_golang/prometheus"
-	kingpin "gopkg.in/alecthomas/kingpin.v2"
 )
 
 const (
@@ -42,18 +41,18 @@ const (
 )
 
 var (
-	unitIncludeSet bool
-	unitInclude    = kingpin.Flag("collector.systemd.unit-include", "Regexp of systemd units to include. Units must both match include and not match exclude to be included.").Default(".+").PreAction(func(c *kingpin.ParseContext) error {
-		unitIncludeSet = true
+	systemdUnitIncludeSet bool
+	systemdUnitInclude    = kingpin.Flag("collector.systemd.unit-include", "Regexp of systemd units to include. Units must both match include and not match exclude to be included.").Default(".+").PreAction(func(c *kingpin.ParseContext) error {
+		systemdUnitIncludeSet = true
 		return nil
 	}).String()
-	oldUnitInclude = kingpin.Flag("collector.systemd.unit-whitelist", "DEPRECATED: Use --collector.systemd.unit-include").Hidden().String()
-	unitExcludeSet bool
-	unitExclude    = kingpin.Flag("collector.systemd.unit-exclude", "Regexp of systemd units to exclude. Units must both match include and not match exclude to be included.").Default(".+\\.(automount|device|mount|scope|slice)").PreAction(func(c *kingpin.ParseContext) error {
-		unitExcludeSet = true
+	oldSystemdUnitInclude = kingpin.Flag("collector.systemd.unit-whitelist", "DEPRECATED: Use --collector.systemd.unit-include").Hidden().String()
+	systemdUnitExcludeSet bool
+	systemdUnitExclude    = kingpin.Flag("collector.systemd.unit-exclude", "Regexp of systemd units to exclude. Units must both match include and not match exclude to be included.").Default(".+\\.(automount|device|mount|scope|slice)").PreAction(func(c *kingpin.ParseContext) error {
+		systemdUnitExcludeSet = true
 		return nil
 	}).String()
-	oldUnitExclude         = kingpin.Flag("collector.systemd.unit-blacklist", "DEPRECATED: Use collector.systemd.unit-exclude").Hidden().String()
+	oldSystemdUnitExclude  = kingpin.Flag("collector.systemd.unit-blacklist", "DEPRECATED: Use collector.systemd.unit-exclude").Hidden().String()
 	systemdPrivate         = kingpin.Flag("collector.systemd.private", "Establish a private, direct connection to systemd without dbus (Strongly discouraged since it requires root. For testing purposes only).").Hidden().Bool()
 	enableTaskMetrics      = kingpin.Flag("collector.systemd.enable-task-metrics", "Enables service unit tasks metrics unit_tasks_current and unit_tasks_max").Bool()
 	enableRestartsMetrics  = kingpin.Flag("collector.systemd.enable-restarts-metrics", "Enables service unit metric service_restart_total").Bool()
@@ -75,9 +74,10 @@ type systemdCollector struct {
 	socketCurrentConnectionsDesc  *prometheus.Desc
 	socketRefusedConnectionsDesc  *prometheus.Desc
 	systemdVersionDesc            *prometheus.Desc
-	unitIncludePattern            *regexp.Regexp
-	unitExcludePattern            *regexp.Regexp
-	logger                        log.Logger
+	// Use regexps for more flexibility than device_filter.go allows
+	systemdUnitIncludePattern *regexp.Regexp
+	systemdUnitExcludePattern *regexp.Regexp
+	logger                    *slog.Logger
 }
 
 var unitStatesName = []string{"active", "activating", "deactivating", "inactive", "failed"}
@@ -87,7 +87,7 @@ func init() {
 }
 
 // NewSystemdCollector returns a new Collector exposing systemd statistics.
-func NewSystemdCollector(logger log.Logger) (Collector, error) {
+func NewSystemdCollector(logger *slog.Logger) (Collector, error) {
 	const subsystem = "systemd"
 
 	unitDesc := prometheus.NewDesc(
@@ -133,26 +133,26 @@ func NewSystemdCollector(logger log.Logger) (Collector, error) {
 		prometheus.BuildFQName(namespace, subsystem, "version"),
 		"Detected systemd version", []string{"version"}, nil)
 
-	if *oldUnitExclude != "" {
-		if !unitExcludeSet {
-			level.Warn(logger).Log("msg", "--collector.systemd.unit-blacklist is DEPRECATED and will be removed in 2.0.0, use --collector.systemd.unit-exclude")
-			*unitExclude = *oldUnitExclude
+	if *oldSystemdUnitExclude != "" {
+		if !systemdUnitExcludeSet {
+			logger.Warn("--collector.systemd.unit-blacklist is DEPRECATED and will be removed in 2.0.0, use --collector.systemd.unit-exclude")
+			*systemdUnitExclude = *oldSystemdUnitExclude
 		} else {
 			return nil, errors.New("--collector.systemd.unit-blacklist and --collector.systemd.unit-exclude are mutually exclusive")
 		}
 	}
-	if *oldUnitInclude != "" {
-		if !unitIncludeSet {
-			level.Warn(logger).Log("msg", "--collector.systemd.unit-whitelist is DEPRECATED and will be removed in 2.0.0, use --collector.systemd.unit-include")
-			*unitInclude = *oldUnitInclude
+	if *oldSystemdUnitInclude != "" {
+		if !systemdUnitIncludeSet {
+			logger.Warn("--collector.systemd.unit-whitelist is DEPRECATED and will be removed in 2.0.0, use --collector.systemd.unit-include")
+			*systemdUnitInclude = *oldSystemdUnitInclude
 		} else {
 			return nil, errors.New("--collector.systemd.unit-whitelist and --collector.systemd.unit-include are mutually exclusive")
 		}
 	}
-	level.Info(logger).Log("msg", "Parsed flag --collector.systemd.unit-include", "flag", *unitInclude)
-	unitIncludePattern := regexp.MustCompile(fmt.Sprintf("^(?:%s)$", *unitInclude))
-	level.Info(logger).Log("msg", "Parsed flag --collector.systemd.unit-exclude", "flag", *unitExclude)
-	unitExcludePattern := regexp.MustCompile(fmt.Sprintf("^(?:%s)$", *unitExclude))
+	logger.Info("Parsed flag --collector.systemd.unit-include", "flag", *systemdUnitInclude)
+	systemdUnitIncludePattern := regexp.MustCompile(fmt.Sprintf("^(?:%s)$", *systemdUnitInclude))
+	logger.Info("Parsed flag --collector.systemd.unit-exclude", "flag", *systemdUnitExclude)
+	systemdUnitExcludePattern := regexp.MustCompile(fmt.Sprintf("^(?:%s)$", *systemdUnitExclude))
 
 	return &systemdCollector{
 		unitDesc:                      unitDesc,
@@ -167,8 +167,8 @@ func NewSystemdCollector(logger log.Logger) (Collector, error) {
 		socketCurrentConnectionsDesc:  socketCurrentConnectionsDesc,
 		socketRefusedConnectionsDesc:  socketRefusedConnectionsDesc,
 		systemdVersionDesc:            systemdVersionDesc,
-		unitIncludePattern:            unitIncludePattern,
-		unitExcludePattern:            unitExcludePattern,
+		systemdUnitIncludePattern:     systemdUnitIncludePattern,
+		systemdUnitExcludePattern:     systemdUnitExcludePattern,
 		logger:                        logger,
 	}, nil
 }
@@ -185,7 +185,7 @@ func (c *systemdCollector) Update(ch chan<- prometheus.Metric) error {
 
 	systemdVersion, systemdVersionFull := c.getSystemdVersion(conn)
 	if systemdVersion < minSystemdVersionSystemState {
-		level.Debug(c.logger).Log("msg", "Detected systemd version is lower than minimum, some systemd state and timer metrics will not be available", "current", systemdVersion, "minimum", minSystemdVersionSystemState)
+		c.logger.Debug("Detected systemd version is lower than minimum, some systemd state and timer metrics will not be available", "current", systemdVersion, "minimum", minSystemdVersionSystemState)
 	}
 	ch <- prometheus.MustNewConstMetric(
 		c.systemdVersionDesc,
@@ -198,16 +198,16 @@ func (c *systemdCollector) Update(ch chan<- prometheus.Metric) error {
 	if err != nil {
 		return fmt.Errorf("couldn't get units: %w", err)
 	}
-	level.Debug(c.logger).Log("msg", "getAllUnits took", "duration_seconds", time.Since(begin).Seconds())
+	c.logger.Debug("getAllUnits took", "duration_seconds", time.Since(begin).Seconds())
 
 	begin = time.Now()
 	summary := summarizeUnits(allUnits)
 	c.collectSummaryMetrics(ch, summary)
-	level.Debug(c.logger).Log("msg", "collectSummaryMetrics took", "duration_seconds", time.Since(begin).Seconds())
+	c.logger.Debug("collectSummaryMetrics took", "duration_seconds", time.Since(begin).Seconds())
 
 	begin = time.Now()
-	units := filterUnits(allUnits, c.unitIncludePattern, c.unitExcludePattern, c.logger)
-	level.Debug(c.logger).Log("msg", "filterUnits took", "duration_seconds", time.Since(begin).Seconds())
+	units := filterUnits(allUnits, c.systemdUnitIncludePattern, c.systemdUnitExcludePattern, c.logger)
+	c.logger.Debug("filterUnits took", "duration_seconds", time.Since(begin).Seconds())
 
 	var wg sync.WaitGroup
 	defer wg.Wait()
@@ -217,7 +217,7 @@ func (c *systemdCollector) Update(ch chan<- prometheus.Metric) error {
 		defer wg.Done()
 		begin = time.Now()
 		c.collectUnitStatusMetrics(conn, ch, units)
-		level.Debug(c.logger).Log("msg", "collectUnitStatusMetrics took", "duration_seconds", time.Since(begin).Seconds())
+		c.logger.Debug("collectUnitStatusMetrics took", "duration_seconds", time.Since(begin).Seconds())
 	}()
 
 	if *enableStartTimeMetrics {
@@ -226,7 +226,7 @@ func (c *systemdCollector) Update(ch chan<- prometheus.Metric) error {
 			defer wg.Done()
 			begin = time.Now()
 			c.collectUnitStartTimeMetrics(conn, ch, units)
-			level.Debug(c.logger).Log("msg", "collectUnitStartTimeMetrics took", "duration_seconds", time.Since(begin).Seconds())
+			c.logger.Debug("collectUnitStartTimeMetrics took", "duration_seconds", time.Since(begin).Seconds())
 		}()
 	}
 
@@ -236,7 +236,7 @@ func (c *systemdCollector) Update(ch chan<- prometheus.Metric) error {
 			defer wg.Done()
 			begin = time.Now()
 			c.collectUnitTasksMetrics(conn, ch, units)
-			level.Debug(c.logger).Log("msg", "collectUnitTasksMetrics took", "duration_seconds", time.Since(begin).Seconds())
+			c.logger.Debug("collectUnitTasksMetrics took", "duration_seconds", time.Since(begin).Seconds())
 		}()
 	}
 
@@ -246,7 +246,7 @@ func (c *systemdCollector) Update(ch chan<- prometheus.Metric) error {
 			defer wg.Done()
 			begin = time.Now()
 			c.collectTimers(conn, ch, units)
-			level.Debug(c.logger).Log("msg", "collectTimers took", "duration_seconds", time.Since(begin).Seconds())
+			c.logger.Debug("collectTimers took", "duration_seconds", time.Since(begin).Seconds())
 		}()
 	}
 
@@ -255,13 +255,13 @@ func (c *systemdCollector) Update(ch chan<- prometheus.Metric) error {
 		defer wg.Done()
 		begin = time.Now()
 		c.collectSockets(conn, ch, units)
-		level.Debug(c.logger).Log("msg", "collectSockets took", "duration_seconds", time.Since(begin).Seconds())
+		c.logger.Debug("collectSockets took", "duration_seconds", time.Since(begin).Seconds())
 	}()
 
 	if systemdVersion >= minSystemdVersionSystemState {
 		begin = time.Now()
 		err = c.collectSystemState(conn, ch)
-		level.Debug(c.logger).Log("msg", "collectSystemState took", "duration_seconds", time.Since(begin).Seconds())
+		c.logger.Debug("collectSystemState took", "duration_seconds", time.Since(begin).Seconds())
 	}
 
 	return err
@@ -273,14 +273,14 @@ func (c *systemdCollector) collectUnitStatusMetrics(conn *dbus.Conn, ch chan<- p
 		if strings.HasSuffix(unit.Name, ".service") {
 			serviceTypeProperty, err := conn.GetUnitTypePropertyContext(context.TODO(), unit.Name, "Service", "Type")
 			if err != nil {
-				level.Debug(c.logger).Log("msg", "couldn't get unit type", "unit", unit.Name, "err", err)
+				c.logger.Debug("couldn't get unit type", "unit", unit.Name, "err", err)
 			} else {
 				serviceType = serviceTypeProperty.Value.Value().(string)
 			}
 		} else if strings.HasSuffix(unit.Name, ".mount") {
 			serviceTypeProperty, err := conn.GetUnitTypePropertyContext(context.TODO(), unit.Name, "Mount", "Type")
 			if err != nil {
-				level.Debug(c.logger).Log("msg", "couldn't get unit type", "unit", unit.Name, "err", err)
+				c.logger.Debug("couldn't get unit type", "unit", unit.Name, "err", err)
 			} else {
 				serviceType = serviceTypeProperty.Value.Value().(string)
 			}
@@ -298,7 +298,7 @@ func (c *systemdCollector) collectUnitStatusMetrics(conn *dbus.Conn, ch chan<- p
 			// NRestarts wasn't added until systemd 235.
 			restartsCount, err := conn.GetUnitTypePropertyContext(context.TODO(), unit.Name, "Service", "NRestarts")
 			if err != nil {
-				level.Debug(c.logger).Log("msg", "couldn't get unit NRestarts", "unit", unit.Name, "err", err)
+				c.logger.Debug("couldn't get unit NRestarts", "unit", unit.Name, "err", err)
 			} else {
 				ch <- prometheus.MustNewConstMetric(
 					c.nRestartsDesc, prometheus.CounterValue,
@@ -316,7 +316,7 @@ func (c *systemdCollector) collectSockets(conn *dbus.Conn, ch chan<- prometheus.
 
 		acceptedConnectionCount, err := conn.GetUnitTypePropertyContext(context.TODO(), unit.Name, "Socket", "NAccepted")
 		if err != nil {
-			level.Debug(c.logger).Log("msg", "couldn't get unit NAccepted", "unit", unit.Name, "err", err)
+			c.logger.Debug("couldn't get unit NAccepted", "unit", unit.Name, "err", err)
 			continue
 		}
 		ch <- prometheus.MustNewConstMetric(
@@ -325,7 +325,7 @@ func (c *systemdCollector) collectSockets(conn *dbus.Conn, ch chan<- prometheus.
 
 		currentConnectionCount, err := conn.GetUnitTypePropertyContext(context.TODO(), unit.Name, "Socket", "NConnections")
 		if err != nil {
-			level.Debug(c.logger).Log("msg", "couldn't get unit NConnections", "unit", unit.Name, "err", err)
+			c.logger.Debug("couldn't get unit NConnections", "unit", unit.Name, "err", err)
 			continue
 		}
 		ch <- prometheus.MustNewConstMetric(
@@ -334,9 +334,7 @@ func (c *systemdCollector) collectSockets(conn *dbus.Conn, ch chan<- prometheus.
 
 		// NRefused wasn't added until systemd 239.
 		refusedConnectionCount, err := conn.GetUnitTypePropertyContext(context.TODO(), unit.Name, "Socket", "NRefused")
-		if err != nil {
-			//log.Debugf("couldn't get unit '%s' NRefused: %s", unit.Name, err)
-		} else {
+		if err == nil {
 			ch <- prometheus.MustNewConstMetric(
 				c.socketRefusedConnectionsDesc, prometheus.GaugeValue,
 				float64(refusedConnectionCount.Value.Value().(uint32)), unit.Name)
@@ -353,7 +351,7 @@ func (c *systemdCollector) collectUnitStartTimeMetrics(conn *dbus.Conn, ch chan<
 		} else {
 			timestampValue, err := conn.GetUnitPropertyContext(context.TODO(), unit.Name, "ActiveEnterTimestamp")
 			if err != nil {
-				level.Debug(c.logger).Log("msg", "couldn't get unit StartTimeUsec", "unit", unit.Name, "err", err)
+				c.logger.Debug("couldn't get unit StartTimeUsec", "unit", unit.Name, "err", err)
 				continue
 			}
 			startTimeUsec = timestampValue.Value.Value().(uint64)
@@ -371,7 +369,7 @@ func (c *systemdCollector) collectUnitTasksMetrics(conn *dbus.Conn, ch chan<- pr
 		if strings.HasSuffix(unit.Name, ".service") {
 			tasksCurrentCount, err := conn.GetUnitTypePropertyContext(context.TODO(), unit.Name, "Service", "TasksCurrent")
 			if err != nil {
-				level.Debug(c.logger).Log("msg", "couldn't get unit TasksCurrent", "unit", unit.Name, "err", err)
+				c.logger.Debug("couldn't get unit TasksCurrent", "unit", unit.Name, "err", err)
 			} else {
 				val = tasksCurrentCount.Value.Value().(uint64)
 				// Don't set if tasksCurrent if dbus reports MaxUint64.
@@ -383,7 +381,7 @@ func (c *systemdCollector) collectUnitTasksMetrics(conn *dbus.Conn, ch chan<- pr
 			}
 			tasksMaxCount, err := conn.GetUnitTypePropertyContext(context.TODO(), unit.Name, "Service", "TasksMax")
 			if err != nil {
-				level.Debug(c.logger).Log("msg", "couldn't get unit TasksMax", "unit", unit.Name, "err", err)
+				c.logger.Debug("couldn't get unit TasksMax", "unit", unit.Name, "err", err)
 			} else {
 				val = tasksMaxCount.Value.Value().(uint64)
 				// Don't set if tasksMax if dbus reports MaxUint64.
@@ -405,7 +403,7 @@ func (c *systemdCollector) collectTimers(conn *dbus.Conn, ch chan<- prometheus.M
 
 		lastTriggerValue, err := conn.GetUnitTypePropertyContext(context.TODO(), unit.Name, "Timer", "LastTriggerUSec")
 		if err != nil {
-			level.Debug(c.logger).Log("msg", "couldn't get unit LastTriggerUSec", "unit", unit.Name, "err", err)
+			c.logger.Debug("couldn't get unit LastTriggerUSec", "unit", unit.Name, "err", err)
 			continue
 		}
 
@@ -477,14 +475,14 @@ func summarizeUnits(units []unit) map[string]float64 {
 	return summarized
 }
 
-func filterUnits(units []unit, includePattern, excludePattern *regexp.Regexp, logger log.Logger) []unit {
+func filterUnits(units []unit, includePattern, excludePattern *regexp.Regexp, logger *slog.Logger) []unit {
 	filtered := make([]unit, 0, len(units))
 	for _, unit := range units {
 		if includePattern.MatchString(unit.Name) && !excludePattern.MatchString(unit.Name) && unit.LoadState == "loaded" {
-			level.Debug(logger).Log("msg", "Adding unit", "unit", unit.Name)
+			logger.Debug("Adding unit", "unit", unit.Name)
 			filtered = append(filtered, unit)
 		} else {
-			level.Debug(logger).Log("msg", "Ignoring unit", "unit", unit.Name)
+			logger.Debug("Ignoring unit", "unit", unit.Name)
 		}
 	}
 
@@ -494,15 +492,15 @@ func filterUnits(units []unit, includePattern, excludePattern *regexp.Regexp, lo
 func (c *systemdCollector) getSystemdVersion(conn *dbus.Conn) (float64, string) {
 	version, err := conn.GetManagerProperty("Version")
 	if err != nil {
-		level.Debug(c.logger).Log("msg", "Unable to get systemd version property, defaulting to 0")
+		c.logger.Debug("Unable to get systemd version property, defaulting to 0")
 		return 0, ""
 	}
 	version = strings.TrimPrefix(strings.TrimSuffix(version, `"`), `"`)
-	level.Debug(c.logger).Log("msg", "Got systemd version", "version", version)
+	c.logger.Debug("Got systemd version", "version", version)
 	parsedVersion := systemdVersionRE.FindString(version)
 	v, err := strconv.ParseFloat(parsedVersion, 64)
 	if err != nil {
-		level.Debug(c.logger).Log("msg", "Got invalid systemd version", "version", version)
+		c.logger.Debug("Got invalid systemd version", "version", version)
 		return 0, ""
 	}
 	return v, version
