@@ -18,14 +18,15 @@ package collector
 
 import (
 	"bufio"
+	"errors"
 	"fmt"
 	"io"
+	"log/slog"
 	"os"
 	"path/filepath"
 	"strconv"
 	"strings"
 
-	"github.com/go-kit/log/level"
 	"github.com/prometheus/client_golang/prometheus"
 )
 
@@ -35,14 +36,74 @@ const (
 	// kstatDataChar   = "0"
 	// kstatDataInt32  = "1"
 	// kstatDataUint32 = "2"
-	// kstatDataInt64  = "3"
+	kstatDataInt64  = "3"
 	kstatDataUint64 = "4"
 	// kstatDataLong   = "5"
 	// kstatDataUlong  = "6"
 	// kstatDataString = "7"
 )
 
-var zfsPoolStatesName = []string{"online", "degraded", "faulted", "offline", "removed", "unavail"}
+var (
+	errZFSNotAvailable = errors.New("ZFS / ZFS statistics are not available")
+
+	zfsPoolStatesName = [...]string{"online", "degraded", "faulted", "offline", "removed", "unavail", "suspended"}
+)
+
+type zfsCollector struct {
+	linuxProcpathBase    string
+	linuxZpoolIoPath     string
+	linuxZpoolObjsetPath string
+	linuxZpoolStatePath  string
+	linuxPathMap         map[string]string
+	logger               *slog.Logger
+}
+
+// NewZFSCollector returns a new Collector exposing ZFS statistics.
+func NewZFSCollector(logger *slog.Logger) (Collector, error) {
+	return &zfsCollector{
+		linuxProcpathBase:    "spl/kstat/zfs",
+		linuxZpoolIoPath:     "/*/io",
+		linuxZpoolObjsetPath: "/*/objset-*",
+		linuxZpoolStatePath:  "/*/state",
+		linuxPathMap: map[string]string{
+			"zfs_abd":         "abdstats",
+			"zfs_arc":         "arcstats",
+			"zfs_dbuf":        "dbufstats",
+			"zfs_dmu_tx":      "dmu_tx",
+			"zfs_dnode":       "dnodestats",
+			"zfs_fm":          "fm",
+			"zfs_vdev_cache":  "vdev_cache_stats", // vdev_cache is deprecated
+			"zfs_vdev_mirror": "vdev_mirror_stats",
+			"zfs_xuio":        "xuio_stats", // no known consumers of the XUIO interface on Linux exist
+			"zfs_zfetch":      "zfetchstats",
+			"zfs_zil":         "zil",
+		},
+		logger: logger,
+	}, nil
+}
+
+func (c *zfsCollector) Update(ch chan<- prometheus.Metric) error {
+	if _, err := c.openProcFile(c.linuxProcpathBase); err != nil {
+		if err == errZFSNotAvailable {
+			c.logger.Debug(err.Error())
+			return ErrNoData
+		}
+	}
+
+	for subsystem := range c.linuxPathMap {
+		if err := c.updateZfsStats(subsystem, ch); err != nil {
+			if err == errZFSNotAvailable {
+				c.logger.Debug(err.Error())
+				// ZFS /proc files are added as new features to ZFS arrive, it is ok to continue
+				continue
+			}
+			return err
+		}
+	}
+
+	// Pool stats
+	return c.updatePoolStats(ch)
+}
 
 func (c *zfsCollector) openProcFile(path string) (*os.File, error) {
 	file, err := os.Open(procFilePath(path))
@@ -50,7 +111,7 @@ func (c *zfsCollector) openProcFile(path string) (*os.File, error) {
 		// file not found error can occur if:
 		// 1. zfs module is not loaded
 		// 2. zfs version does not have the feature with metrics -- ok to ignore
-		level.Debug(c.logger).Log("msg", "Cannot open file for reading", "path", procFilePath(path))
+		c.logger.Debug("Cannot open file for reading", "path", procFilePath(path))
 		return nil, errZFSNotAvailable
 	}
 	return file, nil
@@ -63,8 +124,15 @@ func (c *zfsCollector) updateZfsStats(subsystem string, ch chan<- prometheus.Met
 	}
 	defer file.Close()
 
-	return c.parseProcfsFile(file, c.linuxPathMap[subsystem], func(s zfsSysctl, v uint64) {
-		ch <- c.constSysctlMetric(subsystem, s, v)
+	return c.parseProcfsFile(file, c.linuxPathMap[subsystem], func(s zfsSysctl, v interface{}) {
+		var valueAsFloat64 float64
+		switch value := v.(type) {
+		case int64:
+			valueAsFloat64 = float64(value)
+		case uint64:
+			valueAsFloat64 = float64(value)
+		}
+		ch <- c.constSysctlMetric(subsystem, s, valueAsFloat64)
 	})
 }
 
@@ -78,7 +146,7 @@ func (c *zfsCollector) updatePoolStats(ch chan<- prometheus.Metric) error {
 		file, err := os.Open(zpoolPath)
 		if err != nil {
 			// this file should exist, but there is a race where an exporting pool can remove the files -- ok to ignore
-			level.Debug(c.logger).Log("msg", "Cannot open file for reading", "path", zpoolPath)
+			c.logger.Debug("Cannot open file for reading", "path", zpoolPath)
 			return errZFSNotAvailable
 		}
 
@@ -100,7 +168,7 @@ func (c *zfsCollector) updatePoolStats(ch chan<- prometheus.Metric) error {
 		file, err := os.Open(zpoolPath)
 		if err != nil {
 			// This file should exist, but there is a race where an exporting pool can remove the files. Ok to ignore.
-			level.Debug(c.logger).Log("msg", "Cannot open file for reading", "path", zpoolPath)
+			c.logger.Debug("Cannot open file for reading", "path", zpoolPath)
 			return errZFSNotAvailable
 		}
 
@@ -119,7 +187,7 @@ func (c *zfsCollector) updatePoolStats(ch chan<- prometheus.Metric) error {
 	}
 
 	if zpoolStatePaths == nil {
-		level.Debug(c.logger).Log("msg", "No pool state files found")
+		c.logger.Debug("No pool state files found")
 		return nil
 	}
 
@@ -127,7 +195,7 @@ func (c *zfsCollector) updatePoolStats(ch chan<- prometheus.Metric) error {
 		file, err := os.Open(zpoolPath)
 		if err != nil {
 			// This file should exist, but there is a race where an exporting pool can remove the files. Ok to ignore.
-			level.Debug(c.logger).Log("msg", "Cannot open file for reading", "path", zpoolPath)
+			c.logger.Debug("Cannot open file for reading", "path", zpoolPath)
 			return errZFSNotAvailable
 		}
 
@@ -144,7 +212,7 @@ func (c *zfsCollector) updatePoolStats(ch chan<- prometheus.Metric) error {
 	return nil
 }
 
-func (c *zfsCollector) parseProcfsFile(reader io.Reader, fmtExt string, handler func(zfsSysctl, uint64)) error {
+func (c *zfsCollector) parseProcfsFile(reader io.Reader, fmtExt string, handler func(zfsSysctl, interface{})) error {
 	scanner := bufio.NewScanner(reader)
 
 	parseLine := false
@@ -163,11 +231,18 @@ func (c *zfsCollector) parseProcfsFile(reader io.Reader, fmtExt string, handler
 
 		// kstat data type (column 2) should be KSTAT_DATA_UINT64, otherwise ignore
 		// TODO: when other KSTAT_DATA_* types arrive, much of this will need to be restructured
-		if parts[1] == kstatDataUint64 {
-			key := fmt.Sprintf("kstat.zfs.misc.%s.%s", fmtExt, parts[0])
+		key := fmt.Sprintf("kstat.zfs.misc.%s.%s", fmtExt, parts[0])
+		switch parts[1] {
+		case kstatDataUint64:
 			value, err := strconv.ParseUint(parts[2], 10, 64)
 			if err != nil {
-				return fmt.Errorf("could not parse expected integer value for %q", key)
+				return fmt.Errorf("could not parse expected unsigned integer value for %q: %w", key, err)
+			}
+			handler(zfsSysctl(key), value)
+		case kstatDataInt64:
+			value, err := strconv.ParseInt(parts[2], 10, 64)
+			if err != nil {
+				return fmt.Errorf("could not parse expected signed integer value for %q: %w", key, err)
 			}
 			handler(zfsSysctl(key), value)
 		}
@@ -226,7 +301,8 @@ func (c *zfsCollector) parsePoolObjsetFile(reader io.Reader, zpoolPath string, h
 	parseLine := false
 	var zpoolName, datasetName string
 	for scanner.Scan() {
-		parts := strings.Fields(scanner.Text())
+		line := scanner.Text()
+		parts := strings.Fields(line)
 
 		if !parseLine && len(parts) == 3 && parts[0] == "name" && parts[1] == "type" && parts[2] == "data" {
 			parseLine = true
@@ -240,7 +316,7 @@ func (c *zfsCollector) parsePoolObjsetFile(reader io.Reader, zpoolPath string, h
 			zpoolPathElements := strings.Split(zpoolPath, "/")
 			pathLen := len(zpoolPathElements)
 			zpoolName = zpoolPathElements[pathLen-2]
-			datasetName = parts[2]
+			datasetName = line[strings.Index(line, parts[2]):]
 			continue
 		}
 
@@ -291,3 +367,73 @@ func (c *zfsCollector) parsePoolStateFile(reader io.Reader, zpoolPath string, ha
 
 	return nil
 }
+
+func (c *zfsCollector) constSysctlMetric(subsystem string, sysctl zfsSysctl, value float64) prometheus.Metric {
+	metricName := sysctl.metricName()
+
+	return prometheus.MustNewConstMetric(
+		prometheus.NewDesc(
+			prometheus.BuildFQName(namespace, subsystem, metricName),
+			string(sysctl),
+			nil,
+			nil,
+		),
+		prometheus.UntypedValue,
+		value,
+	)
+}
+
+func (c *zfsCollector) constPoolMetric(poolName string, sysctl zfsSysctl, value uint64) prometheus.Metric {
+	metricName := sysctl.metricName()
+
+	return prometheus.MustNewConstMetric(
+		prometheus.NewDesc(
+			prometheus.BuildFQName(namespace, "zfs_zpool", metricName),
+			string(sysctl),
+			[]string{"zpool"},
+			nil,
+		),
+		prometheus.UntypedValue,
+		float64(value),
+		poolName,
+	)
+}
+
+func (c *zfsCollector) constPoolObjsetMetric(poolName string, datasetName string, sysctl zfsSysctl, value uint64) prometheus.Metric {
+	metricName := sysctl.metricName()
+
+	return prometheus.MustNewConstMetric(
+		prometheus.NewDesc(
+			prometheus.BuildFQName(namespace, "zfs_zpool_dataset", metricName),
+			string(sysctl),
+			[]string{"zpool", "dataset"},
+			nil,
+		),
+		prometheus.UntypedValue,
+		float64(value),
+		poolName,
+		datasetName,
+	)
+}
+
+func (c *zfsCollector) constPoolStateMetric(poolName string, stateName string, isActive uint64) prometheus.Metric {
+	return prometheus.MustNewConstMetric(
+		prometheus.NewDesc(
+			prometheus.BuildFQName(namespace, "zfs_zpool", "state"),
+			"kstat.zfs.misc.state",
+			[]string{"zpool", "state"},
+			nil,
+		),
+		prometheus.GaugeValue,
+		float64(isActive),
+		poolName,
+		stateName,
+	)
+}
+
+type zfsSysctl string
+
+func (s zfsSysctl) metricName() string {
+	parts := strings.Split(string(s), ".")
+	return strings.Replace(parts[len(parts)-1], "-", "_", -1)
+}
